"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ipfs_pubsub_room_1 = __importDefault(require("ipfs-pubsub-room"));
const RemoteProxy_1 = require("../utils/RemoteProxy");
const MockIpfs_1 = require("./MockIpfs");
const ServiceWrapper_1 = require("../utils/ServiceWrapper");
const dc_logging_1 = require("@daocasino/dc-logging");
const dc_configs_1 = require("@daocasino/dc-configs");
const DEFAULT_PEER_TIMEOUT = dc_configs_1.config.default.waitForPeerTimeout;
const logger = new dc_logging_1.Logger("Libp2pTransportProvider");
class Libp2pTransportProvider {
    // static _ipfsNodes: Ipfs[] = []
    constructor(ipfsNode, options) {
        this._options = Object.assign({ waitForPeers: true }, options);
        this._ipfsNode = ipfsNode;
        this.peerId = ipfsNode.id;
        this._roomsMap = new Map();
    }
    async _leaveRoom(address) {
        const room = this._roomsMap.get(address);
        if (room) {
            await room.leave();
            this._roomsMap.delete(address);
            return true;
        }
        return false;
    }
    async _leaveRooms() {
        for (const room of this._roomsMap.values()) {
            await room.leave();
        }
        this._roomsMap.clear();
    }
    async waitForPeer(address, peerId, timeout = DEFAULT_PEER_TIMEOUT) {
        return new Promise((resolve, reject) => {
            const timer = setTimeout(() => {
                reject(new Error("Waiting for peer timed out"));
            }, timeout);
            this._getIpfsRoom(address).once("peer joined", id => {
                if (!peerId || peerId === id) {
                    clearTimeout(timer);
                    resolve();
                }
            });
        });
    }
    static async create() {
        const ipfsNode = await MockIpfs_1.createMockIpfsNode();
        return new Libp2pTransportProvider(ipfsNode);
    }
    async destroy() {
        await this._leaveRooms();
        await MockIpfs_1.destroyMockIpfsNode(this._ipfsNode);
        this._ipfsNode = null;
    }
    _getIpfsRoom(address, name) {
        let room = this._roomsMap.get(address);
        if (!room) {
            room = ipfs_pubsub_room_1.default(this._ipfsNode, address, {})
                .on("error", error => {
                logger.error(error);
            })
                .on("peer joined", id => {
                const roomName = `${name || ""} ${address}`;
                logger.debug(`Peer joined ${id} to ${this._ipfsNode.id} in room ${roomName}`);
            });
            logger.debug(`Room started ${address}`);
            this._roomsMap.set(address, room);
        }
        return room;
    }
    async emitRemote(address, peerId, eventName, params) {
        const room = this._roomsMap.get(address);
        if (!room) {
            throw new Error(`No open room at ${address}`);
        }
        const eventMessage = {
            id: RemoteProxy_1.getId(),
            eventName,
            params: [params],
            from: this.peerId
        };
        try {
            await room.sendTo(peerId, JSON.stringify(eventMessage));
        }
        catch (error) {
            throw error;
        }
    }
    getPeerId() {
        return this.peerId;
    }
    getRemoteInterface(address, roomName) {
        const ipfsRoom = this._getIpfsRoom(address, `Remote interface ${roomName || ""}`);
        const proxy = new RemoteProxy_1.RemoteProxy();
        const self = this;
        ipfsRoom.on("message", message => {
            if (message.from !== self._ipfsNode.id) {
                proxy.onMessage(JSON.parse(message.data));
            }
        });
        return this.waitForPeer(address).then(() => {
            const proxyInterface = proxy.getProxy(message => {
                ipfsRoom.broadcast(JSON.stringify(message));
            });
            // const res: any = { v: proxyInterface }
            return Promise.resolve(proxyInterface);
        });
    }
    exposeSevice(address, service, isEventEmitter = false) {
        const ipfsRoom = this._getIpfsRoom(address, `Expose service ${(service.constructor && service.constructor.name) ||
            ""}`);
        const self = this;
        const wrapper = new ServiceWrapper_1.ServiceWrapper(service, async (response) => {
            const { from, eventName } = response;
            if (eventName) {
                await ipfsRoom.broadcast(JSON.stringify(response));
            }
            else {
                if (from !== self._ipfsNode.id) {
                    try {
                        await ipfsRoom.sendTo(from, JSON.stringify(response));
                    }
                    catch (error) {
                        throw error;
                    }
                }
            }
        }, isEventEmitter);
        if (wrapper.serviceIsEventEmitter) {
            ipfsRoom.on("peer joined", id => {
                service.emit("connected", { id, address });
            });
            ipfsRoom.on("peer left", id => {
                service.emit("disconnected", { id, address });
            });
        }
        ipfsRoom.on("message", message => {
            const { from } = message;
            if (from !== self._ipfsNode.id) {
                const data = JSON.parse(message.data);
                if (data.method) {
                    wrapper.onRequest(Object.assign({}, data, { from }));
                }
            }
        });
    }
    async stopService(address) {
        return this._leaveRoom(address);
    }
}
exports.Libp2pTransportProvider = Libp2pTransportProvider;
//# sourceMappingURL=Libp2pTransportProvider.js.map