"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dc_logging_1 = require("@daocasino/dc-logging");
let _id = 0;
exports.getId = () => {
    return Date.now() * 1000000 + _id++;
};
class ServiceWrapper {
    constructor(service, sendResponse, isEventEmitter = false) {
        this._service = service;
        this.serviceIsEventEmitter = false;
        this.onRequest = this.onRequest.bind(this);
        this.sendResponse = sendResponse.bind(this);
        if (isEventEmitter) {
            this.wrapEventEmitter();
        }
        const loggerName = (service.constructor && service.constructor.name) || "ServiceWrapper";
        this.logger = new dc_logging_1.Logger(loggerName);
    }
    wrapEventEmitter() {
        const onFunc = this._service.on;
        const eventNamesFunc = this._service.eventNames;
        const emitFunc = this._service.emit;
        if (onFunc &&
            typeof onFunc === "function" &&
            emitFunc &&
            typeof emitFunc === "function" &&
            eventNamesFunc &&
            typeof eventNamesFunc === "function") {
            const eventNames = eventNamesFunc.call(this._service);
            eventNames.forEach(eventName => {
                onFunc.call(this._service, eventName, (...params) => {
                    const eventMessage = {
                        id: exports.getId(),
                        eventName,
                        params
                    };
                    this.sendResponse(eventMessage);
                });
            });
            this.serviceIsEventEmitter = true;
        }
    }
    async onRequest(message) {
        const { method, params, id, from } = message;
        const func = this._service[method];
        const response = {
            from,
            id,
            error: null,
            result: null
        };
        if (!method) {
            response.error = {
                status: "ERROR",
                message: "Method not specified"
            };
        }
        if (method.substring(0, 1) === "_") {
            response.error = {
                status: "ERROR",
                message: "Cannot call private function"
            };
        }
        if (typeof func !== "function") {
            response.error = {
                status: "ERROR",
                message: `No function ${method} in ${this._service.constructor.name}`
            };
        }
        if (!response.error) {
            try {
                this.logger.debug(`call method ${method}`);
                response.result = await func.call(this._service, ...params);
            }
            catch (error) {
                this.logger.debug(`response.error method ${method}`);
                response.error = { status: "ERROR", message: error.message };
            }
        }
        if (response.error) {
            this.logger.error(response.error);
        }
        this.sendResponse(response);
    }
}
exports.ServiceWrapper = ServiceWrapper;
//# sourceMappingURL=ServiceWrapper.js.map